import {createDriverAPI} from '@servicenow/ui-testing-helpers';
import {createNowDropdownListDriver} from './now-dropdown-list.driver';
import {createNowPopoverDriver} from '@servicenow/now-popover/drivers';

export const createNowDropdownCustomTargetDriver = createDriverAPI(
	'now-dropdown-custom-target',
	async ({element, driver}) => {
		const popoverEl = await driver.getNodes('>>> now-popover');
		const popover = await createNowPopoverDriver(popoverEl, element);

		return {
			panel: new Proxy(
				{},
				{
					get: function(target, prop) {
						// For any driver.panel.* call, intercept that so that we can
						// dynamically create a new dropdown-list driver instance. This is
						// needed because popover will dynamically hoist or unhoist the
						// panel and trying to reuse a driver after hoist changes prevents
						// elements from being selected properly.
						//
						// async functions will automatically try to access a `then` prop
						// to see if the value is a promise. Need to return undefined
						// because the `driver.panel` is already a
						// resolved value without a `.then()` method on it.
						if (prop !== 'then') {
							return async (...args) => {
								if (prop === 'isDisplayed') {
									return await popover.isPanelDisplayed();
								}
								const panelEl = await driver.getNodes('>>> now-dropdown-list');
								const panel = await createNowDropdownListDriver(
									panelEl,
									element
								);
								if (typeof panel[prop] === 'function') {
									return panel[prop](...args);
								} else {
									throw new TypeError(
										`panel driver does not have a method named '${prop}'`
									);
								}
							};
						}
					}
				}
			)
		};
	}
);
