import {createDriverAPI} from '@servicenow/ui-testing-helpers';
import {createNowDropdownListDriver} from './now-dropdown-list.driver';
import {createNowIconDriver} from '@servicenow/now-icon/drivers';
import {createNowPopoverDriver} from '@servicenow/now-popover/drivers';

export const createNowDropdownDriver = createDriverAPI(
	'now-dropdown',
	async ({find, element, driver}) => {
		const popoverEl = await driver.getNodes('>>> now-popover');
		const popover = await createNowPopoverDriver(popoverEl, element);

		const __private = {
			async getTrigger() {
				return find('button');
			},

			async getTriggerIcon() {
				const presenceEl = await driver.getNodes('>>> now-icon-presence');

				const iconEl = await driver.getNodes(
					'>>> now-icon:not([icon=caret-down-fill])'
				);

				return createNowIconDriver(presenceEl || iconEl, element);
			},

			async getTriggerLabel() {
				return find('button .now-line-height-crop');
			}
		};

		return {
			panel: new Proxy(
				{},
				{
					get: function(target, prop) {
						// For any driver.panel.* call, intercept that so that we can
						// dynamically create a new dropdown-list driver instance. This is
						// needed because popover will dynamically hoist or unhoist the
						// panel and trying to reuse a driver after hoist changes prevents
						// elements from being selected properly.
						//
						// async functions will automatically try to access a `then` prop
						// to see if the value is a promise. Need to return undefined
						// because the `driver.panel` is already a
						// resolved value without a `.then()` method on it.
						if (prop !== 'then') {
							return async (...args) => {
								if (prop === 'isDisplayed') {
									return await popover.isPanelDisplayed();
								}
								const panelEl = await driver.getNodes('>>> now-dropdown-list');
								const panel = await createNowDropdownListDriver(
									panelEl,
									element
								);
								if (typeof panel[prop] === 'function') {
									return panel[prop](...args);
								} else {
									throw new TypeError(
										`panel driver does not have a method named '${prop}'`
									);
								}
							};
						}
					}
				}
			),
			__private,

			/**
			 * Clicks the dropdown trigger element
			 */
			async clickTrigger() {
				return (await __private.getTrigger()).click();
			},

			/**
			 * Returns the text content of the dropdown trigger
			 */
			async getText() {
				return (await __private.getTriggerLabel()).getText();
			}
		};
	}
);
